@@ -8,17 +8,19 @@ functionality.
 
 The data-driven approach is providing a help system to your app
 which allows to add, edit and remove help texts without the need
-to change a single line of code. This gets achieved by overriding the
-:meth:`~ae.gui_app.MainAppBase.change_flow` method.
+to change a single line of code. This gets achieved within
+:class:`HelpAppBase` by overriding the main app class methods
+:meth:`~ae.gui_app.MainAppBase.change_flow` and
+:meth:`~ae.gui_app.MainAppBase.change_app_state`.
 
-For help support on any functionality that is changing an
-app state variable you only need to replace the call of the
-:meth:`~ae.gui_app.MainAppBase.change_app_state` in your app
-code with the method :meth:`~HelpAppBase.help_app_state_change`.
+So for to add help support to the widgets of your app you only
+need to add/provide the help texts with a help id that is
+matching the value of the :attr:`ae_help_id` attribute of
+the widget you need help for.
 
-Finally you only need to add/provide the help texts, which is
-done via an separate i18n translation message file for each
-of the supported languages.
+Additionally you can provide a separate i18n translation message
+file for each of the supported languages for to make your
+help texts multi-lingual.
 
 
 help layout implementation example
@@ -51,19 +53,18 @@ abstract methods :meth:`~ae.gui_app.MainAppBase.init_app` and
 
 For to activate the help mode the widget
 for to display the help texts have to be assigned to
-the attribute :attr:`~HelpAppBase.ae_help_layout`::
+the attribute :attr:`~HelpAppBase.help_layout`::
 
-    main_app.ae_help_layout = HelpScreenContainerOrWindow()
+    main_app.help_layout = HelpScreenContainerOrWindow()
 
-So :attr:`~HelpAppBase.ae_help_layout is also used as a flag
+So :attr:`~HelpAppBase.help_layout is also used as a flag
 of the help mode activity. By assigning `None` to this
 attribute the help mode will be deactivated::
 
-    main_app.ae_help_layout = None
+    main_app.help_layout = None
 
-Optionally you can use the attribute
-:attr:`~HelpAppBase.ae_help_activator` for
-to store a widget that allows the user
+Use the attribute :attr:`~HelpAppBase.help_activator`
+for to provide and store a widget that allows the user
 to toggle the help mode activation. The
 :meth:`~HelpAppBase.help_display` is using
 it as fallback widget if no help target
@@ -77,8 +78,9 @@ widget got found.
     This more complete example is also demonstrating
     the implementation and usage of the help activator
     and layout widgets (in the classes
-    :class:`~ae.kivy_app.HelpToggler` and
-    :class:`~ae.kivy_app.HelpLayout`).
+    :class:`~ae.kivy_help.HelpBehaviour`,
+    :class:`~ae.kivy_help.HelpToggler` and
+    :class:`~ae.kivy_help.HelpLayout`).
 
 
 additional helper functions
@@ -86,7 +88,7 @@ additional helper functions
 
 The helper functions :func:`anchor_points`, :func:`layout_ps_hints`,
 :func:`layout_x` and :func:`layout_y`, provided
-by this module, are covering the framework-independent calculation
+by this module, are providing a framework-independent calculation
 of the position and size of the help layout and the anchor.
 
 
@@ -95,7 +97,8 @@ flow change context message id
 
 The message id for to identify the help texts for each flow
 button is composed by the :meth:`~HelpAppBase.help_flow_id`,
-using the prefix marker string `'help_flow#'` followed by
+using the prefix marker string defined by the module variable
+:data:`HELP_ID_PREFIX_FLOW` followed by
 the flow id of the flow widget.
 
 .. hint::
@@ -125,7 +128,8 @@ application state change context message id
 -------------------------------------------
 
 The message ids for app state change help texts are using
-the prefix marker string `'help_app_state#'` followed by
+the prefix marker string defined by the module variable
+:data:`HELP_ID_PREFIX_STATE`, followed by
 the name of the app state and are composed via the
 :meth:`~HelpAppBase.help_app_state_id`.
 
@@ -134,18 +138,12 @@ pluralize-able help texts
 -------------------------
 
 Each message id can optionally have several different
-help texts for their pluralization. For to support
-them simply pass an `count` argument to either
-:meth:`~HelpAppBase.change_flow`
-:paramref:`~HelpAppBase.change_flow.event_kwargs`
-argument or to the
-:paramref:`~HelpAppBase.help_app_state_change.count`
-argument of the method
-:meth:`~HelpAppBase.help_app_state_change`.
-
-With that you can define a help text for the
-following count cases in your message text file
-like this::
+help texts for their pluralization. For that
+simply add a `count` item to the `ae_help_vars`
+property of the help target widget and then
+define a help text for the all the possible
+count cases in your message text file
+like shown in the following example::
 
     {
         'message_id': {
@@ -158,6 +156,12 @@ like this::
        ...
     }
 
+.. hint::
+    The provided `count` value can also be
+    included/displayed in the help text,
+    like shown in the `'zero'` count case of
+    the example.
+
 
 pre- and post-change help texts
 -------------------------------
@@ -177,6 +181,24 @@ example::
     }
 
 
+If you want to move/change the help target to
+another widget after a change, then use instead of
+`'after'` the '`next_help_id'` message dict key::
+
+    {
+        'message_id': {
+                       '':              "help text before the change",
+                       'next_help_id':  "help_flow#next_flow_id",
+                       },
+       ...
+    }
+
+In this case the help target will automatically change
+to the widget specified by the flow id in the '`next_help_id'`
+key, if the user was tapping the second time on the
+first/initial help target widget.
+
+
 i18n help texts
 ---------------
 
@@ -206,16 +228,18 @@ repository of the
 from abc import abstractmethod, ABC
 from typing import Any, Dict, Optional, Sequence, Tuple, Union
 
-from ae.inspector import stack_vars, try_eval                                   # type: ignore
-from ae.i18n import default_language, get_f_string, translation                 # type: ignore
-from ae.gui_app import FLOW_KEY_SEP, id_of_flow, MainAppBase                    # type: ignore
+from ae.inspector import stack_vars                                                         # type: ignore
+from ae.i18n import default_language, get_f_string, translation                             # type: ignore
+from ae.gui_app import FLOW_KEY_SEP, module_globals, id_of_flow, MainAppBase                # type: ignore
 
 
-__version__ = '0.1.6'
+__version__ = '0.1.7'
 
 
-HELP_ID_FLOW_PREFIX = 'help_flow#'
-IGNORED_HELP_FLOWS = (id_of_flow('close', 'flow_popup'), )  #: tuple of flow ids never search/show help text for
+HELP_ID_PREFIX_FLOW = 'help_flow#'
+HELP_ID_PREFIX_STATE = 'help_app_state#'
+IGNORED_HELP_FLOWS = (id_of_flow('close', 'flow_popup'), )          #: tuple of flow ids never search/show help text for
+IGNORED_HELP_STATES = ('flow_id', 'flow_path', 'win_rectangle')     #: tuple of app state names never searched help for
 
 
 def anchor_points(radius: float, anchor_x: float, anchor_y: float, anchor_dir: str) -> Tuple[float, ...]:
@@ -288,7 +312,7 @@ def layout_ps_hints(wid_x: float, wid_y, wid_width, wid_height, win_width, win_h
         anchor_y = wid_y + (0 if anchor_dir_y == 'd' else wid_height)
         max_width = win_width
 
-    return locals()
+    return locals().copy()
 
 
 def layout_x(anchor_x: float, anchor_dir: str, width: float, win_width: float) -> float:
@@ -327,9 +351,9 @@ class HelpAppBase(MainAppBase, ABC):
     """ main app help base class. """
 
     # additional instance attributes
-    ae_help_activator: Any = None           #: help flow mode de-/activator button widget
-    ae_help_id: str = ''                    #: message id of the currently explained/focused target widget in help mode
-    ae_help_layout: Optional[Any] = None    #: container/popup/dropdown/window widget in active help flow mode else None
+    displayed_help_id: str = ''         #: message id of the currently explained/focused target widget in help mode
+    help_activator: Any = None          #: help mode de-/activator button widget
+    help_layout: Optional[Any] = None   #: container/popup/dropdown/window widget in active help mode else None
 
     @abstractmethod
     def ensure_top_most_z_index(self, widget: Any):
@@ -371,53 +395,53 @@ class HelpAppBase(MainAppBase, ABC):
                                 the list :data:`ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION`).
         """
         count = event_kwargs.pop('count', None)
-        help_args = locals()
-        if not self.help_flow_display(help_args, count=count):
+        help_vars = dict(new_flow_id=new_flow_id, popups_to_close=popups_to_close, event_kwargs=event_kwargs)
+        if count is not None:
+            help_vars['count'] = count
+
+        if not self.help_flow_display(help_vars):
             if super().change_flow(new_flow_id, popups_to_close=popups_to_close, **event_kwargs):
-                return self.help_flow_display(help_args, changed=True, count=count)
+                self.help_flow_display(help_vars, changed=True)
+                return True
         return False
 
-    def help_app_state_change(self, state_name: str, new_value: Any,
-                              send_event: bool = True, count: Optional[int] = None):
+    def change_app_state(self, state_name: str, state_value: Any, send_event: bool = True):
         """ change app state via :meth:`~ae.gui_app.MainAppBase.change_app_state`, show help text in active help mode.
 
         :param state_name:      name of the app state to change.
-        :param new_value:       new value of the app state to change.
+        :param state_value:     new value of the app state to change.
         :param send_event:      pass False to prevent to send/call the on_<state_name> event to the main app instance.
-        :param count:           optional item count for pluralized help texts.
         """
-        help_vars = locals()
-        if not self.help_app_state_display(help_vars, count=count):
-            self.change_app_state(state_name, new_value, send_event=send_event)
-            self.help_app_state_display(help_vars, changed=True, count=count)
+        help_vars = dict(state_name=state_name, state_value=state_value, send_event=send_event)
+        if not self.help_app_state_display(help_vars):
+            super().change_app_state(state_name, state_value, send_event=send_event)
+            self.help_app_state_display(help_vars, changed=True)
 
-    def help_app_state_display(self, help_vars: Dict[str, Any], changed: bool = False, count: Optional[int] = None
-                               ) -> bool:
+    def help_app_state_display(self, help_vars: Dict[str, Any], changed: bool = False) -> bool:
         """ actualize the help layout if active, before and after the change of the app state.
 
         :param help_vars:       locals (args/kwargs) of overwritten :meth:`~ae.gui_app.MainAppBase.change_flow` method.
-        :param changed:         False before change to new flow, pass True if flow got changed already.
-        :param count:           item count for pluralized help texts.
-        :return:                return value for caller (:meth:`~ae.gui_app.MainAppBase.change_flow`) which is
-                                True if help layout is active and got updated, else False.
+
+                                items passed to the help text renderer:
+                                    * `count`: optional number used for to render a pluralized help text
+                                      for this app state change.
+
+        :param changed:         False before change of the app state, pass True if app state got just/already changed.
+        :return:                True if help mode and layout is active and found target widget is locked, else False.
         """
-        hlw = self.ae_help_layout
+        hlw = self.help_layout
         if hlw is None:
-            return changed              # inactive help layout
+            return False            # inactive help layout
 
-        widget = None
         state_name = help_vars['state_name']
+        if state_name in IGNORED_HELP_STATES:
+            return False
+
         help_id = self.help_app_state_id(state_name)
-        if help_id == self.ae_help_id:
-            if not changed:
-                return False            # allow to execute app state change
-        else:
-            widget = self.help_widget('ae_state_name', state_name, help_vars)
-
-        key_suffix = 'after' if changed else ''
-        self.help_display(help_id, help_vars, widget, count=count, key_suffix=key_suffix, must_have=not changed)
-
-        return True
+        ret = self.help_display(help_id, help_vars, key_suffix='after' if changed else '')
+        if help_id == self.displayed_help_id and not changed:
+            ret = False             # allow to execute app state change
+        return ret
 
     @staticmethod
     def help_app_state_id(state_name: str) -> str:
@@ -426,81 +450,88 @@ class HelpAppBase(MainAppBase, ABC):
         :param state_name:      name of the app state variable.
         :return:                help id for the specified app state.
         """
-        return f'help_app_state#{state_name}'
+        return f'{HELP_ID_PREFIX_STATE}{state_name}'
 
     def help_display(self, help_id: str, help_vars: Dict[str, Any], widget: Optional[Any] = None,
-                     count: Optional[int] = None, key_suffix: str = '', must_have: bool = False):
-        """ display help text to the user in activated help flow mode.
+                     key_suffix: str = '', must_have: bool = False) -> bool:
+        """ display help text to the user in activated help mode.
 
         :param help_id:         help id to show help text for.
         :param help_vars:       variables used in the conversion of the f-string expression to a string.
+                                items passed to the help text renderer:
+                                    * `count`: optional number used for to render a pluralized help text.
         :param widget:          target widget for to show help text for.
-        :param count:           pass if the translated text changes on pluralization (see :func:`get_text`).
-                                If passed then the value of this argument will be provided/overwritten in the
-                                globals as a variable with the name `count`.
         :param key_suffix:      suffix to the key used if the translation is a dict.
         :param must_have:       pass True for to display error help text and console output if no help text exists.
+        :return:                True if help text got found and displayed.
         """
-        hlw: Any = self.ae_help_layout
-        hlw.widget = widget or self.ae_help_activator
-
-        self.change_observable('ae_help_id', help_id)
-
         has_trans = translation(help_id)
+        short_help_id = help_id
         if not has_trans and FLOW_KEY_SEP in help_id:
-            help_id = help_id[:help_id.index(FLOW_KEY_SEP)]  # remove detail (e.g. flow key or app state value)
-            has_trans = translation(help_id)
+            short_help_id = help_id[:help_id.index(FLOW_KEY_SEP)]  # remove detail (e.g. flow key or app state value)
+            has_trans = translation(short_help_id)
 
-        if must_have and not has_trans:
+        if not has_trans:
+            if not must_have:
+                return False
             if self.debug:
-                hlw.help_text = f"help id [b]'{help_id}'[/b] has no\ntranslated help text in '{default_language()}'"
+                help_id = f"No translation found for help id [b]'{help_id}/{key_suffix}'[/b] in '{default_language()}'"
             else:
                 help_id = ''        # show at least initial help text as fallback
-                has_trans = True
+            short_help_id = help_id
+            key_suffix = ''
             self.play_beep()
+        elif key_suffix == 'after' and 'next_help_id' in has_trans:
+            help_id = short_help_id = has_trans['next_help_id']
+            key_suffix = ''
 
-        if has_trans:
-            help_vars['app'] = self.framework_app
-            help_vars['main_app'] = self
-            hlw.help_text = get_f_string(help_id, count=count, key_suffix=key_suffix,
-                                         glo_vars=try_eval('globals()'), loc_vars=help_vars)
+        wid = None
+        if help_id:     # if not in init/reset then ensure target widget and extend help_vars with call stack locals
+            wid = self.help_widget('ae_help_id', help_id, help_vars)
+            if not wid:
+                wid = self.widget_by_attribute('ae_help_id', help_id)
+                if wid:
+                    help_vars['self'] = wid
+                elif widget and widget not in (wid, self.help_activator):
+                    self.dpo(f"HelpAppBase.help_display(): target={wid}/{help_id} missing or IS NOT event_wid={widget}")
+        widget = wid or widget or self.help_activator
+
+        hlw: Any = self.help_layout
+        hlw.target = widget
+        self.change_observable('displayed_help_id', help_id)
+
+        glo_vars = module_globals.copy()
+        glo_vars.update(globals())
+        glo_vars['_add_base_globals'] = True
+        help_vars['app'] = self.framework_app
+        help_vars['main_app'] = self
+        hlw.help_text = get_f_string(short_help_id, key_suffix=key_suffix, glo_vars=glo_vars, loc_vars=help_vars)
 
         self.ensure_top_most_z_index(hlw)
+        self.call_method('on_help_displayed')
 
-    def help_flow_display(self, help_vars: Dict[str, Any], changed: bool = False, count: Optional[int] = None) -> bool:
+        return True
+
+    def help_flow_display(self, help_vars: Dict[str, Any], changed: bool = False) -> bool:
         """ actualize the help layout if active, exclusively called by :meth:`~ae.gui_app.MainAppBase.change_flow`.
 
         :param help_vars:       locals (args/kwargs) of overwritten :meth:`~ae.gui_app.MainAppBase.change_flow` method.
         :param changed:         False before change to new flow, pass True if flow got changed already.
-        :param count:           item count for pluralized help texts.
-        :return:                return value for caller (:meth:`~ae.gui_app.MainAppBase.change_flow`) which is
-                                True if help layout is active and got updated, else False.
+        :return:                True if help layout is active and found target widget is locked, else False.
         """
-        hlw = self.ae_help_layout
+        hlw = self.help_layout
         if hlw is None:
-            return changed              # inactive help layout
+            return False            # inactive help layout
 
         nfi = help_vars['new_flow_id']
         if nfi in IGNORED_HELP_FLOWS:
-            if not changed:     # check help target widget is still visible and if not then show initial help text
-                wid = self.widget_by_flow_id(self.help_id_flow())
-                if wid is None or not getattr(wid, 'visible', True):
-                    self.help_display('', dict())
-            return changed
+            return False
 
         help_id = self.help_flow_id(nfi)
-        if help_id == self.ae_help_id:
-            if not changed:
-                return False            # allow to execute flow change of currently explained flow button
-            widget = hlw.widget
-            self.help_widget('ae_flow_id', nfi, help_vars)  # extend help_vars also for 'changed/after' help text
-        else:
-            widget = self.help_widget('ae_flow_id', nfi, help_vars)
-
-        key_suffix = 'after' if changed else ''
-        self.help_display(help_id, help_vars, widget, count=count, key_suffix=key_suffix, must_have=not changed)
-
-        return True
+        ret = self.help_display(help_id, help_vars, key_suffix='after' if changed else '', must_have=not changed)
+        if help_id == self.displayed_help_id and not changed:
+            ret = False             # allow to execute flow change of currently explained flow button
+        return ret
 
     @staticmethod
     def help_flow_id(flow_id: str) -> str:
@@ -509,25 +540,25 @@ class HelpAppBase(MainAppBase, ABC):
         :param flow_id:         flow id to show help text for.
         :return:                help id for the specified flow id and suffix.
         """
-        return f'{HELP_ID_FLOW_PREFIX}{flow_id}'
+        return f'{HELP_ID_PREFIX_FLOW}{flow_id}'
 
     def help_id_flow(self) -> str:
         """ determine flow id of the current help id. """
-        hid = self.ae_help_id
-        if hid.startswith(HELP_ID_FLOW_PREFIX):
-            return hid[len(HELP_ID_FLOW_PREFIX):]
+        hid = self.displayed_help_id
+        if hid.startswith(HELP_ID_PREFIX_FLOW):
+            return hid[len(HELP_ID_PREFIX_FLOW):]
         return ''
 
     def help_widget(self, attr_name: str, attr_value: str, help_vars: Dict[str, Any]) -> Optional[Any]:
-        """ get help target widget via variable name/value and extend :paramref:`~help_widget.help_vars` locals.
+        """ find help target widget via attribute name/value and extend :paramref:`~help_widget.help_vars`.
 
-        :param attr_name:       widget attribute name for to detect widget and declaration locals.
-        :param attr_value:      widget attribute value for to detect widget and declaration locals
-        :param help_vars:       help flow variables, to be extended with event activation stack frame locals.
+        :param attr_name:       widget attribute name for to detect widget and call stack locals.
+        :param attr_value:      widget attribute value for to detect widget and call stack locals
+        :param help_vars:       help env variables, to be extended with event activation stack frame locals.
         :return:                found help target widget or None if not found.
         """
         depth = 3
-        while depth <= 12:
+        while depth <= 15:
             # look for FlowButton widget in kv/enaml rule call stack frame for translation text context
             _gfv, lfv, _deep = stack_vars('do not skip ae_gui_app', min_depth=depth, max_depth=depth)
             widget = lfv.get('self')
